#include "OggPlugin.h"

#include <stdlib.h>
#include <string.h>

ogg_t* ogg_open(char *path)
{
	ogg_t *ogg;
	FILE *f;

	if((ogg = (ogg_t *) calloc(1, sizeof(ogg_t))) == NULL )
	{
		return(NULL);
	}

	if((f = fopen(path,"rb")) == NULL )
	{
		free(ogg);
		return(NULL);
	}

	if(ov_open(f, ogg, NULL, 0) < 0) {
		fclose(f);
		free(ogg);
		return(NULL);
	}

	return(ogg);
}

int ogg_check_sig(char *path)
{
	ogg_t *ogg;
	FILE *f;

	if((ogg = (ogg_t *) calloc(1, sizeof(ogg_t))) == NULL )
	{
		return(0);
	}

	if((f = fopen(path,"rb")) == NULL )
	{
		free(ogg);
		return(0);
	}

	if(ov_test(f, ogg, NULL, 0) != 0) {
		fclose(f);
		free(ogg);
		return(0);
	}

	ov_clear(ogg);
	free(ogg);
	return(1);
}

int ogg_close(ogg_t *file)
{
	ov_clear(file);
	free(file);
	return(0);
}
/*
samples is the number of samples to be got so each sample is 2 bytes long and there are 2 channels
*/
int ogg_read_audio(ogg_t *file, short *output_i, long samples, int stream)
{
	long result;
	int dummy;
	FILE *d;
	int loop = 0;	/* loop is in bytes */
	int bytelen = samples * 2;

	d = fopen("debug.txt","a");

	fprintf(d,"read_audio: samples=%d bytelen=%d\n", samples, bytelen);
	while( bytelen > 0 )
	{
		/* assume that the buffer can hold samples length and each sample is stereo */
		result = ov_read(file,(char *) output_i+loop,bytelen,0,2,1,&dummy);
		if(result == 0)
		{
			fprintf(d,"EOF loop=%d\n", loop );
			break;
		}
		loop += result;
		bytelen -= result;
		fprintf(d,"loop=%d bytelen=%d result=%d stream=%d\n", loop, bytelen, result, dummy);
	}
	fclose(d);
	return(0);
}

int ogg_streams(ogg_t *file)
{
	return(ov_streams(file));
}

int ogg_channels(ogg_t *file, int stream)
{
	vorbis_info *vi;

	if((vi = ov_info(file, stream)) == NULL )
	{
		return(0);
	}

	return( vi->channels );
}

long ogg_sampleRate(ogg_t *file, int stream)
{
	vorbis_info *vi;

	if((vi = ov_info(file, stream)) == NULL )
	{
		return(0);
	}

	return( vi->rate );
}

long ogg_samples(ogg_t *file, int stream)
{

	return(ov_pcm_total(file, stream));

}

long ogg_current_sample(ogg_t *file, int stream)
{

	return(ov_pcm_tell(file));

}

int ogg_set_sample(ogg_t *file, long sample, int stream)
{
	return(ov_pcm_seek(file, sample));
}